@@ -17,12 +17,13 @@ module Criterion.Main.Options
     , MatchType(..)
     , defaultConfig
     , parseWith
+    , config
     , describe
     , versionInfo
     ) where
 
--- Temporary: to support pre-AMP GHC 7.8.4:
-import Data.Monoid 
+-- Temporary: to support pre-SMP
+import Data.Semigroup ((<>))
 
 import Control.Monad (when)
 import Criterion.Analysis (validateAccessors)
@@ -39,16 +40,27 @@ import Options.Applicative.Help (Chunk(..), tabulate)
 import Options.Applicative.Help.Pretty ((.$.))
 import Options.Applicative.Types
 import Paths_criterion (version)
+import Prelude ()
+import Prelude.Compat
+import Statistics.Types (mkCL,cl95)
 import Text.PrettyPrint.ANSI.Leijen (Doc, text)
 import qualified Data.Map as M
-import Prelude
 
 -- | How to match a benchmark name.
 data MatchType = Prefix
                  -- ^ Match by prefix. For example, a prefix of
                  -- @\"foo\"@ will match @\"foobar\"@.
                | Glob
-                 -- ^ Match by Unix-style glob pattern.
+                 -- ^ Match by Unix-style glob pattern. When using this match
+                 -- type, benchmark names are treated as if they were
+                 -- file-paths. For example, the glob patterns @\"*/ba*\"@ and
+                 -- @\"*/*\"@ will match @\"foo/bar\"@, but @\"*\"@ or @\"*bar\"@
+                 -- __will not__.
+               | Pattern
+                 -- ^ Match by searching given substring in benchmark
+                 -- paths.
+               | IPattern
+                 -- ^ Same as 'Pattern', but case insensitive.
                deriving (Eq, Ord, Bounded, Enum, Read, Show, Typeable, Data,
                          Generic)
 
@@ -57,7 +69,7 @@ data Mode = List
             -- ^ List all benchmarks.
           | Version
             -- ^ Print the version.
-          | RunIters Int64 MatchType [String]
+          | RunIters Config Int64 MatchType [String]
             -- ^ Run the given benchmarks, without collecting or
             -- analysing performance numbers.
           | Run Config MatchType [String]
@@ -67,8 +79,7 @@ data Mode = List
 -- | Default benchmarking configuration.
 defaultConfig :: Config
 defaultConfig = Config {
-      confInterval = 0.95
-    , forceGC      = True
+      confInterval = cl95
     , timeLimit    = 5
     , resamples    = 1000
     , regressions  = []
@@ -86,29 +97,38 @@ parseWith :: Config
              -- ^ Default configuration to use if options are not
              -- explicitly specified.
           -> Parser Mode
-parseWith cfg =
-    (matchNames (Run <$> config cfg)) <|>
-    runIters <|>
-    (List <$ switch (long "list" <> short 'l' <> help "List benchmarks")) <|>
-    (Version <$ switch (long "version" <> help "Show version info"))
+parseWith cfg = config cfg <**> runMode
+                -- Important: only run `config` once here, as we only want the
+                -- command-line options resulting from `config` to appear once
+                -- in the `--help` output. See #168.
   where
-    runIters = matchNames $
-      RunIters <$> option auto
-                  (long "iters" <> short 'n' <> metavar "ITERS" <>
-                   help "Run benchmarks, don't analyse")
+    runMode :: Parser (Config -> Mode)
+    runMode =
+      matchNames (pure $ \mt bs cfg' -> Run cfg' mt bs) <|>
+      runIters <|>
+      (const List <$ switch (long "list" <> short 'l' <> help "List benchmarks")) <|>
+      (const Version <$ switch (long "version" <> help "Show version info"))
+
+    runIters :: Parser (Config -> Mode)
+    runIters = matchNames $ (\iters mt bs cfg' -> RunIters cfg' iters mt bs)
+      <$> option auto
+          (long "iters" <> short 'n' <> metavar "ITERS" <>
+           help "Run benchmarks, don't analyse")
+
+    matchNames :: Parser (MatchType -> [String] -> Config -> Mode)
+               -> Parser (Config -> Mode)
     matchNames wat = wat
       <*> option match
           (long "match" <> short 'm' <> metavar "MATCH" <> value Prefix <>
-           help "How to match benchmark names (\"prefix\" or \"glob\")")
+           help "How to match benchmark names (\"prefix\", \"glob\", \"pattern\", or \"ipattern\")")
       <*> many (argument str (metavar "NAME..."))
 
+-- | Parse a configuration.
 config :: Config -> Parser Config
 config Config{..} = Config
-  <$> option (range 0.001 0.999)
+  <$> option (mkCL <$> range 0.001 0.999)
       (long "ci" <> short 'I' <> metavar "CI" <> value confInterval <>
        help "Confidence interval")
-  <*> (not <$> switch (long "no-gc" <> short 'G' <>
-                       help "Do not collect garbage between iterations"))
   <*> option (range 0.1 86400)
       (long "time-limit" <> short 'L' <> metavar "SECS" <> value timeLimit <>
        help "Time limit to run a benchmark")
@@ -154,12 +174,14 @@ match :: ReadM MatchType
 match = do
   m <- readerAsk
   case map toLower m of
-    mm | mm `isPrefixOf` "pfx"    -> return Prefix
-       | mm `isPrefixOf` "prefix" -> return Prefix
-       | mm `isPrefixOf` "glob"   -> return Glob
-       | otherwise                ->
-         readerError $ show m ++ " is not a known match type. "
-                              ++ "Try \"prefix\" or \"glob\"."
+    mm | mm `isPrefixOf` "pfx"      -> return Prefix
+       | mm `isPrefixOf` "prefix"   -> return Prefix
+       | mm `isPrefixOf` "glob"     -> return Glob
+       | mm `isPrefixOf` "pattern"  -> return Pattern
+       | mm `isPrefixOf` "ipattern" -> return IPattern
+       | otherwise                  -> readerError $
+                                       show m ++ " is not a known match type"
+                                              ++ "Try \"prefix\", \"pattern\", \"ipattern\" or \"glob\"."
 
 regressParams :: ReadM ([String], String)
 regressParams = do
