@@ -17,15 +17,14 @@ module Criterion.Monad.Internal
     , Crit(..)
     ) where
 
--- Temporary: to support pre-AMP GHC 7.8.4:
-import Control.Applicative 
-
+import Control.Monad.Catch (MonadThrow, MonadCatch, MonadMask)
 import Control.Monad.Reader (MonadReader(..), ReaderT)
 import Control.Monad.Trans (MonadIO)
 import Criterion.Types (Config)
 import Data.IORef (IORef)
+import Prelude ()
+import Prelude.Compat
 import System.Random.MWC (GenIO)
-import Prelude
 
 data Crit = Crit {
     config   :: !Config
@@ -36,7 +35,7 @@ data Crit = Crit {
 -- | The monad in which most criterion code executes.
 newtype Criterion a = Criterion {
       runCriterion :: ReaderT Crit IO a
-    } deriving (Functor, Applicative, Monad, MonadIO)
+    } deriving (Functor, Applicative, Monad, MonadIO, MonadThrow, MonadCatch, MonadMask)
 
 instance MonadReader Config Criterion where
     ask     = config `fmap` Criterion ask
